# Config file for rc_genicam_api
#
# It defines the following variables:
#
# RC_GENICAM_API_INCLUDE_DIRS     - include directory
# RC_GENICAM_API_LIBRARIES        - all dynamic libraries
# RC_GENICAM_API_STATIC_LIBRARIES - all static libraries


####### Expanded from @PACKAGE_INIT@ by configure_package_config_file() #######
####### Any changes to this file will be overwritten by the next CMake run ####
####### The input file was PROJECTConfig.cmake.in                            ########

get_filename_component(PACKAGE_PREFIX_DIR "${CMAKE_CURRENT_LIST_DIR}/../../../" ABSOLUTE)

macro(set_and_check _var _file)
  set(${_var} "${_file}")
  if(NOT EXISTS "${_file}")
    message(FATAL_ERROR "File or directory ${_file} referenced by variable ${_var} does not exist !")
  endif()
endmacro()

macro(check_required_components _NAME)
  foreach(comp ${${_NAME}_FIND_COMPONENTS})
    if(NOT ${_NAME}_${comp}_FOUND)
      if(${_NAME}_FIND_REQUIRED_${comp})
        set(${_NAME}_FOUND FALSE)
      endif()
    endif()
  endforeach()
endmacro()

####################################################################################

set_and_check(RC_GENICAM_API_INCLUDE_DIRS "${PACKAGE_PREFIX_DIR}/include")

### Resolve Genicam first
function(resolve_genicam)
  set_and_check(GENICAM_INCLUDE_DIR "${PACKAGE_PREFIX_DIR}/include/rc_genicam_api/genicam")
  set_and_check(GENICAM_LIBRARY_DIR "${PACKAGE_PREFIX_DIR}/lib/x86_64-linux-gnu")

  list(APPEND RC_GENICAM_API_INCLUDE_DIRS "${GENICAM_INCLUDE_DIR}")
  set(RC_GENICAM_API_INCLUDE_DIRS "${RC_GENICAM_API_INCLUDE_DIRS}" PARENT_SCOPE)

  if (UNIX)
    # try to get architecture from compiler
    EXECUTE_PROCESS(COMMAND ${CMAKE_CXX_COMPILER} -dumpmachine COMMAND tr -d '\n' OUTPUT_VARIABLE CXX_MACHINE)
    string(REGEX REPLACE "([a-zA-Z_0-9]+).*" "\\1" ARCHITECTURE ${CXX_MACHINE})
  elseif (WIN32)
    if ("${CMAKE_SIZEOF_VOID_P}" STREQUAL "4")
      set(ARCHITECTURE WIN32_i86)
    else ()
      set(ARCHITECTURE WIN64_x64)
    endif ()
  endif ()

  message(STATUS "Detected architecture ${ARCHITECTURE}")

  set(GENICAM_LIB_SUFFIX)
  if ("${ARCHITECTURE}" STREQUAL "arm")
    set(GENICAM_LIB_SUFFIX ".so")
  elseif ("${ARCHITECTURE}" STREQUAL "aarch64")
    set(GENICAM_LIB_SUFFIX ".so")
  elseif ("${ARCHITECTURE}" STREQUAL "i686")
    set(GENICAM_LIB_SUFFIX ".so")
  elseif ("${ARCHITECTURE}" STREQUAL "x86_64")
    set(GENICAM_LIB_SUFFIX ".so")
  elseif ("${ARCHITECTURE}" STREQUAL "WIN32_i86")
    set(GENICAM_LIB_SUFFIX ".lib")
  elseif ("${ARCHITECTURE}" STREQUAL "WIN64_x64")
    set(GENICAM_LIB_SUFFIX ".lib")
  else ()
    message(FATAL_ERROR "Unknown architecture")
  endif ()

  foreach(GENICAM_LIB genicam::libGCBase_gcc48_v3_4;genicam::libGenApi_gcc48_v3_4;genicam::liblog4cpp_gcc48_v3_4;genicam::libLog_gcc48_v3_4;genicam::libMathParser_gcc48_v3_4;genicam::libNodeMapData_gcc48_v3_4;genicam::libXmlParser_gcc48_v3_4)
    add_library(${GENICAM_LIB} SHARED IMPORTED)
    set_property(TARGET ${GENICAM_LIB} APPEND PROPERTY IMPORTED_CONFIGURATIONS DEBUG)
    set_property(TARGET ${GENICAM_LIB} APPEND PROPERTY IMPORTED_CONFIGURATIONS RELEASE)

    string(REPLACE "genicam::" "" GENICAM_LIB_NAME ${GENICAM_LIB})
    set(GENICAM_LIB_FILE "${GENICAM_LIBRARY_DIR}/${GENICAM_LIB_NAME}${GENICAM_LIB_SUFFIX}")

    list(APPEND RC_GENICAM_API_LIBRARIES "${GENICAM_LIB_FILE}")
    set(RC_GENICAM_API_LIBRARIES "${RC_GENICAM_API_LIBRARIES}" PARENT_SCOPE)

    if (UNIX)
      set_target_properties(${GENICAM_LIB}
         PROPERTIES
           INTERFACE_INCLUDE_DIRECTORIES "${GENICAM_INCLUDE_DIR}"
           IMPORTED_LOCATION_DEBUG "${GENICAM_LIB_FILE}"
           IMPORTED_LOCATION_RELEASE "${GENICAM_LIB_FILE}"
           IMPORTED_LOCATION "${GENICAM_LIB_FILE}")
    elseif (WIN32)
      set_target_properties(${GENICAM_LIB}
        PROPERTIES
          INTERFACE_INCLUDE_DIRECTORIES "${GENICAM_INCLUDE_DIR}"
          IMPORTED_IMPLIB_DEBUG  "${GENICAM_LIB_FILE}"
          IMPORTED_IMPLIB_RELEASE "${GENICAM_LIB_FILE}"
          IMPORTED_IMPLIB "${GENICAM_LIB_FILE}")
    endif ()
  endforeach()
endfunction(resolve_genicam)

resolve_genicam()

## Targets
include("${CMAKE_CURRENT_LIST_DIR}/RC_GENICAM_APITargets.cmake")

set(RC_GENICAM_API_STATIC_LIBRARIES )
list(APPEND RC_GENICAM_API_LIBRARIES rc_genicam_api)

